﻿using System;
using System.Threading;
using Ozeki.VoIP;

namespace _05_Instant_Messages
{
    class Program
    {
        private static Softphone _mySoftphone;
        private static InstantMessage _messageToSend;

        static void Main(string[] args)
        {
            InitSoftphone();
            ShowGreetingMessage();
            ReadRegisterInfos();

            // prevents the termination of the application
            BlockExit();
        }


        private static void InitSoftphone()
        {
            _mySoftphone = new Softphone();
            _mySoftphone.PhoneLineStateChanged += mySoftphone_PhoneLineStateChanged;
            _mySoftphone.IncomingMessage += mySoftphone_IncomingMessage;
        }


        /// <summary>
        /// Receiving incoming messages from other lines by the event.
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private static void mySoftphone_IncomingMessage(object sender, InstantMessage e)
        {
            Console.WriteLine("\nMessage received from {0}: {1}", e.Sender, e.Content);
            MessageToSend();
        }


        private static void mySoftphone_PhoneLineStateChanged(object sender, RegistrationStateChangedArgs e)
        {
            Console.WriteLine("Phone line state changed: {0}", e.State);

            if (e.State == RegState.Error || e.State == RegState.NotRegistered)
            {
                ReadRegisterInfos();
            }
            else if (e.State == RegState.RegistrationSucceeded)
            {
                Console.WriteLine("Registration succeeded - ONLINE");

                // Gets the recipient first
                GetRecipient();

                // Than ready to send messages
                MessageToSend();
            }
        }


        /// <summary>
        /// Gets the recipient from the user.
        /// </summary>
        private static void GetRecipient()
        {
            while (string.IsNullOrEmpty(_mySoftphone.Recipient))
            {
                Console.Write("\nPlease enter the number of the recipient: ");
                _mySoftphone.Recipient = Console.ReadLine();
            }
            Console.WriteLine();
        }

        
        /// <summary>
        /// Method to send instant messages
        /// </summary>
        private static void MessageToSend()
        {
            while (true)
            {
                var content = GetMessage();
                _messageToSend = new InstantMessage(_mySoftphone.Recipient, content);
                _mySoftphone.SendMessage(_messageToSend);
            }
        }


        /// <summary>
        /// Gets the message from the user
        /// </summary>
        private static string GetMessage()
        {
            Console.Write("Message to {0}: ", _mySoftphone.Recipient);
            return Console.ReadLine();
        }


        /// <summary>
        /// Shows a greeting message with information about the example application.
        /// </summary>
        private static void ShowGreetingMessage()
        {
            Console.WriteLine("This is a simple Ozeki VoIP SIP SDK demo, written in C#.");
            Console.WriteLine("This example is about to introduce the followings:");
            Console.WriteLine("- sending Instant Messages");
            Console.WriteLine("- receiving Instant Messages");
            Console.WriteLine("-------------------------------------------------------------------------------");
            Console.WriteLine();
        }


        private static void ReadRegisterInfos()
        {
            var registrationRequired = true;
            Console.WriteLine("\nPlease set up Your SIP account:\n");

            // Asks, if a registration is required to the PBX. The default value is true.
            Console.Write("Please set if the registration is required (true/false) (default: true): ");
            var regRequired = Read("Registration required", false);
            if (regRequired.ToLower() == "false" || regRequired.ToLower() == "no" ||
                     regRequired.ToLower() == "n")
            {
                registrationRequired = false;
            }
            else
            {
                Console.WriteLine("Registration set to required.");
            }


            // The SIP account needs and authentication ID, and some names aswell.
            Console.Write("Please set Your authentication ID: ");
            var authenticationId = Read("Authentication ID", true);

            // If the user only presses the Enter button, the username will be the same as the authentication ID
            Console.Write("Please set Your username (default: " + authenticationId + "): ");
            var userName = Read("Username", false);
            if (string.IsNullOrEmpty(userName))
                userName = authenticationId;

            // If the user only presses the Enter button, the display name will be the same as the authentication ID
            Console.Write("Please set Your name to be displayed (default: " + authenticationId + "): ");
            var displayName = Read("Display name", false);
            if (string.IsNullOrEmpty(displayName))
                displayName = authenticationId;

            // The registration password needs to be entered.
            Console.Write("Please set Your registration password: ");
            var registerPassword = Read("Password", true);

            // Domain name as a string, for example an IP adress.
            Console.Write("Please set the domain name: ");
            var domainHost = Read("Domain name", true);

            // Port number with the as 5060 default value.
            Console.Write("Please set the port number (default: 5060): ");
            int domainPort;
            string port = Read("Port", false);
            if (string.IsNullOrEmpty(port))
            {
                domainPort = 5060;
            }
            else
            {
                domainPort = int.Parse(port);
            }
            Console.WriteLine("\nCreating SIP account and trying to register....\n");

            // When every information has been given, we are trying to register to the PBX with the softphone's Register() method.
            _mySoftphone.Register(registrationRequired, displayName, userName, authenticationId, registerPassword,
                                 domainHost, domainPort);
        }


        /// <summary>
        /// Reads data from user input. (Helper method.)
        /// </summary>
        /// <param name="inputName"></param>
        /// <param name="readWhileEmpty"></param>
        /// <returns></returns>
        private static string Read(string inputName, bool readWhileEmpty)
        {
            while (true)
            {
                string input = Console.ReadLine();

                if (!readWhileEmpty)
                    return input;

                if (!string.IsNullOrEmpty(input))
                    return input;

                Console.WriteLine(inputName + " cannot be empty.");
                Console.Write(inputName + ": ");
            }
        }
        
        /// <summary>
        /// Doesn't let the application to exit.
        /// </summary>
        private static void BlockExit()
        {
            while (true)
            {
                Thread.Sleep(10);
            }
        }
    }
}
